@@ -1,30 +1,32 @@
-{-# LANGUAGE OverloadedStrings, NoMonomorphismRestriction #-}
-module Data.Greskell.GraphSONSpec (main,spec) where
+{-# LANGUAGE CPP                       #-}
+{-# LANGUAGE NoMonomorphismRestriction #-}
+{-# LANGUAGE OverloadedStrings         #-}
+module Data.Greskell.GraphSONSpec
+    ( main
+    , spec
+    ) where
 
-import Data.Aeson (object, (.=), ToJSON(..), FromJSON(..), Value(..))
-import qualified Data.Aeson as Aeson
-import Data.Aeson.Types (parseEither, Value(..), Parser)
-import qualified Data.ByteString.Lazy as BSL
-import Data.Either (isLeft)
-import Data.HashMap.Strict (HashMap)
-import qualified Data.HashMap.Strict as HM
-import Data.List (isInfixOf)
-import Data.Int (Int32)
-import Data.Monoid ((<>))
-import Data.Text (Text)
-import qualified Data.Vector as V
-import Test.Hspec
+import           Data.Aeson                    (FromJSON (..), ToJSON (..), Value (..), object,
+                                                (.=))
+import qualified Data.Aeson                    as Aeson
+import qualified Data.Aeson.KeyMap             as KM
+import           Data.Aeson.Types              (Parser, Value (..), parseEither)
+import qualified Data.ByteString.Lazy          as BSL
+import           Data.Either                   (isLeft)
+import           Data.HashMap.Strict           (HashMap)
+import qualified Data.HashMap.Strict           as HM
+import           Data.Int                      (Int32)
+import           Data.List                     (isInfixOf)
+import           Data.Monoid                   ((<>))
+import           Data.Text                     (Text)
+import qualified Data.Vector                   as V
+import           Test.Hspec
 
-import Data.Greskell.GMap (GMapEntry(..), unGMapEntry)
-import Data.Greskell.GraphSON
-  ( GraphSON, parseTypedGraphSON,
-    -- parseTypedGraphSON',
-    typedGraphSON', nonTypedGraphSON,
-    nonTypedGValue, typedGValue',
-    GValue, GValueBody(..),
-    FromGraphSON(..)
-  )
-import Data.Greskell.GraphSON.GValue (unwrapAll, unwrapOne)
+import           Data.Greskell.GMap            (GMapEntry (..), unGMapEntry)
+import           Data.Greskell.GraphSON        (FromGraphSON (..), GValue, GValueBody (..),
+                                                GraphSON, nonTypedGValue, nonTypedGraphSON,
+                                                parseTypedGraphSON, typedGValue', typedGraphSON')
+import           Data.Greskell.GraphSON.GValue (unwrapAll, unwrapOne)
 
 main :: IO ()
 main = hspec spec
@@ -76,6 +78,16 @@ fromJSON_spec = describe "FromJSON (recursive)" $ do
                        ]
     parse input `shouldBe` Right (nonTypedGraphSON input)
 
+isParseTypeError :: String -- ^ expected type
+                 -> String -- ^ error message
+                 -> Bool -- ^ matched
+#if MIN_VERSION_aeson(1,4,3)
+isParseTypeError exp_type = isInfixOf ("parsing " <> exp_type <> " failed")
+#else
+isParseTypeError exp_type = isInfixOf ("expected " <> exp_type)
+#endif
+
+
 parseTypedGraphSON_spec :: Spec
 parseTypedGraphSON_spec = describe "parseTypedGraphSON" $ do
   let parse = parseEither parseTypedGraphSON
@@ -91,7 +103,7 @@ parseTypedGraphSON_spec = describe "parseTypedGraphSON" $ do
       got `shouldSatisfy` (isInfixOf "Expected @type")
     specify "typed JSON with valid @type field and invalid @value" $ do
       let (Left got) = parse (object ["@type" .= String "g:Int32", "@value" .= String "hoge"]) :: Either String (GraphSON Int32)
-      got `shouldSatisfy` (isInfixOf "expected Int32")
+      got `shouldSatisfy` isParseTypeError "Int32"
     specify "typed JSON with valid @type and @value" $ do
       let got = parse (object ["@type" .= String "g:Int32", "@value" .= Number 255]) :: Either String (GraphSON Int32)
       got `shouldBe` (Right $ typedGraphSON' "g:Int32" 255)
@@ -111,7 +123,7 @@ parseTypedGraphSON_spec = describe "parseTypedGraphSON" $ do
       let (Left got) = parse (object [ "@type" .= String "g:Array",
                                        "@value" .= Number 100
                                      ]) :: Either String (GraphSON (HashMap String String))
-      got `shouldSatisfy` (isInfixOf "expected HashMap")
+      got `shouldSatisfy` isParseTypeError "HashMap"
     specify "typed JSON with valid @type and @value" $ do
       let got = parse (object [ "@type" .= String "g:Map",
                                 "@value" .= object ["hoge" .= String "HOGE", "foo" .= String "FOO"]
@@ -179,9 +191,9 @@ gvalue_spec = do
     fromToJSON "wrapped array" (gson "g:List" "[null, \"foo\", 100]")
       (wrapped "g:List" $ GArray $ fmap bare $ V.fromList [GNull, GString "foo", GNumber 100])
     fromToJSON "bare object" "{\"foo\": \"bar\", \"hoge\": 99, \"quux\": null}"
-      (bare $ GObject $ fmap bare $ HM.fromList [("foo", GString "bar"), ("hoge", GNumber 99), ("quux", GNull)])
+      (bare $ GObject $ fmap bare $ KM.fromList [("foo", GString "bar"), ("hoge", GNumber 99), ("quux", GNull)])
     fromToJSON "wrapped object" (gson "g:Map" "{\"foo\": \"bar\", \"hoge\": 99, \"quux\": null}")
-      (wrapped "g:Map" $ GObject $ fmap bare $ HM.fromList [("foo", GString "bar"), ("hoge", GNumber 99), ("quux", GNull)])
+      (wrapped "g:Map" $ GObject $ fmap bare $ KM.fromList [("foo", GString "bar"), ("hoge", GNumber 99), ("quux", GNull)])
     nested_spec
     double_wrap_spec
     decode_error_spec
@@ -198,7 +210,7 @@ nested_spec :: Spec
 nested_spec = fromToJSON "mixed nested" nestedSample expected
   where
     expected = wrapped "g:List" $ GArray $ V.fromList [ bare $ GNumber 100, exp_a, exp_b ]
-    exp_a = wrapped "g:Map" $ GObject $ HM.fromList
+    exp_a = wrapped "g:Map" $ GObject $ KM.fromList
             [ ("foo", bare $ GNumber 100),
               ("bar", wrapped "g:Int" $ GNumber 200)
             ]
@@ -208,7 +220,7 @@ nested_spec = fromToJSON "mixed nested" nestedSample expected
               exp_c,
               wrapped "g:Boolean" $ GBool False
             ]
-    exp_c = bare $ GObject $ HM.fromList
+    exp_c = bare $ GObject $ KM.fromList
             [ ("xxx", wrapped "g:Int" $ GNumber 200),
               ("yyy", bare $ GBool True)
             ]
@@ -220,7 +232,7 @@ double_wrap_spec :: Spec
 double_wrap_spec = fromToJSON "double wrapped" input expected
   where
     input = gson "g:Object" $ gson "g:Int" "100"
-    expected = wrapped "g:Object" $ GObject $ HM.fromList
+    expected = wrapped "g:Object" $ GObject $ KM.fromList
                [ ("@type", bare $ GString "g:Int"),
                  ("@value", bare $ GNumber 100)
                ]
@@ -241,7 +253,7 @@ decode_error_spec = do
 forceDecode :: FromJSON a => BSL.ByteString -> a
 forceDecode json = case Aeson.eitherDecode json of
   Left err -> error ("Unexpected decode failure: " <> err)
-  Right a -> a
+  Right a  -> a
 
 fromToJSON :: String -> BSL.ByteString -> GValue -> Spec
 fromToJSON label input_json expected = specify label $ do
@@ -252,7 +264,7 @@ fromToJSON label input_json expected = specify label $ do
     decoded = forceDecode input_json
     encoded = Aeson.toJSON decoded
     exp_enc = forceDecode input_json
-    
+
 bare :: GValueBody -> GValue
 bare = nonTypedGValue
 
@@ -264,7 +276,7 @@ gson ftype fvalue = "{\"@type\":\"" <> ftype <> "\",\"@value\":" <> fvalue <> "}
 
 fromLeft' :: (Show a, Show b) => Either a b -> a
 fromLeft' (Left a) = a
-fromLeft' e = error ("Expecting Left, but got " ++ show e)
+fromLeft' e        = error ("Expecting Left, but got " ++ show e)
 
 unwrap_spec :: Spec
 unwrap_spec = do
